/*****************************************************************************
!                              INTEL CONFIDENTIAL
! Copyright(C) 2003-2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       MKL DFTI interface example program (C-interface)
!
!       Backward complex 1D transform for double precision data inplace.
!
!*****************************************************************************
! Configuration parameters:
!           DFTI_FORWARD_DOMAIN = DFTI_COMPLEX (obligatory)
!           DFTI_PRECISION      = DFTI_DOUBLE  (obligatory)
!           DFTI_DIMENSION      = 1            (obligatory)
!           DFTI_LENGTHS        = n            (obligatory)
!           DFTI_PLACEMENT      = DFTI_INPLACE (default)
!           DFTI_BACKWARD_SCALE = 1.0/n        (default=1.0)
!
! Other default configuration parameters are in the mkl_dfti.h interface file
!****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_dfti_examples.h"

int main(int argc, char *argv[])  /* COMPLEX_1D_DOUBLE_EX9 */
{
    mkl_double_complex* x_in;
    mkl_double_complex* x_exp;

    DFTI_DESCRIPTOR_HANDLE hand = 0;
    MKL_LONG n;

    MKL_LONG Status;
    double Scale;
    int i;
    double maxerr, d;
    double eps = DOUBLE_EPS;
    int failure = 0;

    /*
    **  Read input data from input file
    **  n - size of transform
    */
    if (read_data_file_1d(argc, argv, &n)) {
        printf(" TEST FAILED : Read input data from input file\n");
        return 1;
    }

    if (LEGEND_PRINT) {
        printf("\n\n COMPLEX_1D_DOUBLE_EX9\n");
        printf(" Backward 1D complex transform for double precision data\n");
        printf(" Accuracy test with sinusoid tone input data\n\n");
        printf(" Configuration parameters:\n\n");
        printf(" DFTI_FORWARD_DOMAIN = DFTI_COMPLEX\n");
        printf(" DFTI_PRECISION      = DFTI_DOUBLE\n");
        printf(" DFTI_DIMENSION      = 1\n");
        printf(" DFTI_LENGTHS        = %d\n", n);
        printf(" DFTI_PLACEMENT      = DFTI_INPLACE\n");
        printf(" DFTI_BACKWARD_SCALE = 1.0/n\n\n");
    }

    /*
    **  Allocate arrays for input and expected data
    */
    x_in  = (mkl_double_complex*)malloc(2*n*sizeof(double));
    x_exp = (mkl_double_complex*)malloc(2*n*sizeof(double));

    /*
    **  Init Sinusoid tone and expected uotput result
    */
    init_backward_tone_and_expected_result_z(x_in, x_exp, n);

    if (ADVANCED_DATA_PRINT) {
        printf(" INPUT vector X_IN\n");
        print_vector_z( x_in, n);
    }

    /*
    **  Create DFTI descriptor for 1D double precision transform
    */
    Status = DftiCreateDescriptor(&hand, DFTI_DOUBLE, DFTI_COMPLEX, 1, n);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiCreateDescriptor(&hand, DFTI_DOUBLE,...\n");
        failure++;
        goto FREE_ARRAYS;
    }

    /*
    **  Set Scale number for Backward transform
    */
    Scale = 1.0/(double)n;
    printf("\n DFTI_BACKWARD_SCALE  = 1/n\n");

    Status = DftiSetValue(hand, DFTI_BACKWARD_SCALE, Scale);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiSetValue(hand, DFTI_BACKWARD_SCALE, Scale)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Commit DFTI descriptor
    */
    Status = DftiCommitDescriptor(hand);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiCommitDescriptor(hand)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Compute Backward transform
    */
    printf("\n Compute DftiComputeBackward\n\n");
    Status = DftiComputeBackward(hand, x_in);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiComputeBackward(hand, x_in)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    if (ADVANCED_DATA_PRINT) {
        printf(" Backward result vector X_IN\n");
        print_vector_z( x_in, n);
    }

    /*
    **  Check result
    */
    maxerr = check_result_z(x_in, x_exp, n);

    if (ACCURACY_PRINT)
        printf("\n Accuracy = %g\n\n", maxerr);

    if (maxerr < eps) {
        printf(" TEST PASSED\n");
    } else {
        printf(" TEST FAILED\n");
        failure++;
    }

    /*
    **  Free DFTI descriptor
    */
FREE_DESCRIPTOR:
    Status = DftiFreeDescriptor(&hand);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiFreeDescriptor(&hand)\n");
        failure++;
    }

    /*
    **  Free arrays for input and expected data
    */
FREE_ARRAYS:
    free(x_in);
    free(x_exp);

    if (failure)
        return 1;

    printf("\n END OF TEST\n");

    return 0;
}
