/*****************************************************************************
!                              INTEL CONFIDENTIAL
! Copyright(C) 2003-2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       MKL DFTI interface example program (C-interface)
!
!       Complex-to-complex multiple 1D transform for single precision data
!       which are allocated in two-dimension array, inplace.
!
!       Perform multiple row one-dimensional transforms.
!       Note that row is along leading dimension and data have unit stride
!
!*****************************************************************************
! Configuration parameters:
!           DFTI_FORWARD_DOMAIN = DFTI_COMPLEX     (obligatory)
!           DFTI_PRECISION      = DFTI_SINGLE      (obligatory)
!           DFTI_DIMENSION      = 1                (obligatory)
!           DFTI_LENGTHS        = n                (obligatory)
!           DFTI_PLACEMENT      = DFTI_INPLACE     (default)
!           DFTI_FORWARD_SCALE  = 1.0              (default)
!           DFTI_BACKWARD_SCALE = 1.0/n            (default=1.0)
!           DFTI_INPUT_STRIDES  = {first_index, 1} (default={0,1})
!           DFTI_NUMBER_OF_TRANSFORMS = multiple   (default=1)
!           DFTI_INPUT_DISTANCE       = dist_in    (obligatory,
!                                              if NUMBER_OF_TRANSFORMS >1)
!
! Other default configuration parameters are in the mkl_dfti.h interface file
!****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "mkl_dfti.h"
#include "mkl_dfti_examples.h"

int main(int argc, char *argv[])  /* COMPLEX_1D_SINGLE_EX3 */
{
    mkl_float_complex x_in [M_MAX][N_MAX];
    mkl_float_complex x_exp[M_MAX][N_MAX];

    DFTI_DESCRIPTOR_HANDLE hand = 0;

    MKL_LONG n;
    MKL_LONG first_index;
    MKL_LONG multiple;

    MKL_LONG Status;
    float Scale;

    MKL_LONG dist_in;
    MKL_LONG strides_in[2];

    float maxerr;
    float eps = SINGLE_EPS;
    int failure = 0;

    /*
    **  Read input data from input file
    **  n - size of transform
    **  first_index - displacement from the first element of data array
    **  multiple - number of multiple transform
    */
    if (read_data_file_1d_mult(argc, argv, &n, &first_index, &multiple)) {
        printf(" TEST FAILED : Read input data from input file\n");
        return 1;
    }

    /*
    ** Put transform parameters
    */
    dist_in  = N_MAX;

    strides_in[0] = first_index;
    strides_in[1] = 1;

    if (LEGEND_PRINT) {
        printf("\n\n COMPLEX_1D_SINGLE_EX3\n");
        printf(" Forward-Backward multiple row 1D complex transform\n");
        printf(" Note that row is along leading dimension and data have unit stride\n\n");
        printf(" Configuration parameters:\n\n");
        printf(" DFTI_FORWARD_DOMAIN = DFTI_COMPLEX\n");
        printf(" DFTI_PRECISION      = DFTI_SINGLE\n");
        printf(" DFTI_DIMENSION      = 1\n");
        printf(" DFTI_LENGTHS        = %d\n", n);
        printf(" DFTI_PLACEMENT      = DFTI_INPLACE\n");
        printf(" DFTI_FORWARD_SCALE  = 1.0\n");
        printf(" DFTI_BACKWARD_SCALE = 1.0/n\n\n");
        printf(" DFTI_INPUT_DISTANCE = %d\n", N_MAX);
        printf(" DFTI_INPUT_STRIDES  = {%d,%d}\n", strides_in[0], strides_in[1]);
        printf(" DFTI_NUMBER_OF_TRANSFORMS = %d\n", multiple);
    }

    /*
    **  Check input parameters
    */
    if (first_index + multiple*n > M_MAX*N_MAX) {
        printf(" Error input parameters: first_index + multiple*n > M_MAX*N_MAX\n");
        printf(" Please see mkl_dfti_examples.h file\n");
        printf(" TEST FAILED\n");
        return 1;
    }

    /*
    **  Put input data and expected result
    */
    zero_init_c(x_in, M_MAX*N_MAX);
    init_multiple_rows_c(x_in, n, multiple, first_index, dist_in);
    cblas_ccopy(M_MAX*N_MAX, x_in, 1, x_exp, 1);

    if (ADVANCED_DATA_PRINT) {
        printf("\n INPUT X by columns (for multiple=3)\n\n");
        print_three_rows_c(x_in, n, first_index, dist_in);
    }

    /*
    **  Create DFTI descriptor for 1D single precision transform
    */
    Status = DftiCreateDescriptor(&hand, DFTI_SINGLE, DFTI_COMPLEX, 1, n);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiCreateDescriptor(&hand, DFTI_SINGLE,...\n");
        return 1;
    }

    /*
    **  Set parameters for multiple transform mode
    */
    if (multiple > 1) {
        Status = DftiSetValue(hand, DFTI_NUMBER_OF_TRANSFORMS, multiple);
        if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
            dfti_example_status_print(Status);
            printf(" TEST FAILED : DftiSetValue(hand, DFTI_NUMBER_OF_TRANSFORMS, multiple)\n");
            failure++;
            goto FREE_DESCRIPTOR;
        }

        Status = DftiSetValue(hand, DFTI_INPUT_DISTANCE, dist_in);
        if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
            dfti_example_status_print(Status);
            printf(" TEST FAILED : DftiSetValue(hand, DFTI_INPUT_DISTANCE, dist_in)\n");
            failure++;
            goto FREE_DESCRIPTOR;
        }
    }

    /*
    **  Set strides parameters
    */
    Status = DftiSetValue(hand, DFTI_INPUT_STRIDES, strides_in);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiSetValue(hand, DFTI_INPUT_STRIDES, strides_in)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Commit DFTI descriptor
    */
    Status = DftiCommitDescriptor(hand);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiCommitDescriptor(hand)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Compute Forward transform
    */
    printf("\n Compute DftiComputeForward\n");
    Status = DftiComputeForward(hand, x_in);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiComputeForward(hand, x_in)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    if (ADVANCED_DATA_PRINT) {
        printf("\n Forward result X by columns (for multiple=3)\n\n");
        print_three_rows_c(x_in, n, first_index, dist_in);
    }

    /*
    **  Set Scale number for Backward transform
    */
    Scale = 1.0/(float)n;
    printf("\n\n DFTI_BACKWARD_SCALE  = 1/n\n");

    Status = DftiSetValue(hand, DFTI_BACKWARD_SCALE, Scale);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiSetValue(hand, DFTI_BACKWARD_SCALE, Scale)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Commit DFTI descriptor
    */
    Status = DftiCommitDescriptor(hand);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiCommitDescriptor(hand)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    /*
    **  Compute Backward transform
    */
    printf("\n Compute DftiComputeBackward\n\n");
    Status = DftiComputeBackward(hand, x_in);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiComputeBackward(hand, x_in)\n");
        failure++;
        goto FREE_DESCRIPTOR;
    }

    if (ADVANCED_DATA_PRINT) {
        printf(" Backward result X by columns (for multiple=3)\n\n");
        print_three_rows_c(x_in, n, first_index, dist_in);
    }

    /*
    **  Check result
    */
    maxerr = check_result_c(x_in, x_exp, M_MAX*N_MAX);

    if (ACCURACY_PRINT)
        printf("\n Accuracy = %g\n\n", maxerr);

    if (maxerr < eps) {
        printf(" TEST PASSED\n");
    } else {
        printf(" TEST FAILED\n");
        failure++;
    }

    /*
    **  Free DFTI descriptor
    */
FREE_DESCRIPTOR:
    Status = DftiFreeDescriptor(&hand);
    if (! DftiErrorClass(Status, DFTI_NO_ERROR)) {
        dfti_example_status_print(Status);
        printf(" TEST FAILED : DftiFreeDescriptor(&hand)\n");
        failure++;
    }

    if (failure)
        return 1;

    printf("\n END OF TEST\n");

    return 0;
}
