/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2006-2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!    MKL Cluster DFT using FFTW interface (via wrappers)
!    example program (C-interface)
!
!    Forward-Backward 1D real-to-complex transform for double/single
!    precision data.
!
!****************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "rfftw_mpi.h"

#ifdef FFTW_ENABLE_FLOAT
    #define EPS 1.0E-6
#else
    #define EPS 1.0E-12
#endif

int main(int argc,char *argv[])
{

    fftw_real *local,*work;
    fftw_real *preal,t,err,g_err,scale;
    int i,PrS,PrR;
    rfftwnd_mpi_plan plan;

    int len,nx,nx_out,start_x,start_x_out,size;
    int failure = 0;

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&PrS);
    MPI_Comm_rank(MPI_COMM_WORLD,&PrR);
    len = 256;
    if (PrR==0) printf("PrS=%d, lengths={%d}\n",PrS,len);

/*--- Usage of MPI FFTW for Real-to-Complex One-dimensional Transforms (work==NULL) ---*/

    if (PrR==0) printf("Usage of MPI FFTW for Real-to-Complex One-dimensional Transforms (work==NULL)\n");
    plan=rfftwnd_mpi_create_plan(MPI_COMM_WORLD,1,&len,FFTW_FORWARD,FFTW_ESTIMATE);
    if (PrR==0) printf("     CreatePlan for Forward....DONE\n");
    rfftwnd_mpi_local_sizes(plan,&nx,&start_x,&nx_out,&start_x_out,&size);
    if (PrR==0) printf("     LocalSizes................DONE\n");
    local=(fftw_real*)fftw_malloc(size*sizeof(fftw_complex));
    preal=(fftw_real*)local;
    srand(PrR*100);
    for (i=0;i<nx;i++)
        preal[i]=(fftw_real)rand()/RAND_MAX;

    rfftwnd_mpi(plan,1,local,NULL,FFTW_NORMAL_ORDER);
    if (PrR==0) printf("     Forward FFTW..............DONE\n");

    rfftwnd_mpi_destroy_plan(plan);
    if (PrR==0) printf("     Destroy plan for Forward..DONE\n");

    plan=rfftwnd_mpi_create_plan(MPI_COMM_WORLD,1,&len,FFTW_BACKWARD,FFTW_ESTIMATE);
    if (PrR==0) printf("     CreatePlan for Backward...DONE\n");

    rfftwnd_mpi(plan,1,local,NULL,FFTW_NORMAL_ORDER);
    scale=1.0/len;
    if (PrR==0) printf("     Backward FFTW.............DONE\n");

    for (i=0;i<nx;i++)
        preal[i]*=scale;

    rfftwnd_mpi_destroy_plan(plan);
    if (PrR==0) printf("     Destroy plan for Backward.DONE\n");
    srand(PrR*100);
    err=0.0;
    for (i=0;i<nx;i++) {
        t=fabs(preal[i]-(fftw_real)rand()/RAND_MAX);
        if (t>err) err=t;
    }
    fftw_free(local);
#ifdef FFTW_ENABLE_FLOAT
    MPI_Reduce(&err,&g_err,1,MPI_FLOAT,MPI_MAX,0,MPI_COMM_WORLD);
#else
    MPI_Reduce(&err,&g_err,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
#endif
    if (PrR==0) {
        printf("     Error=%e\n",g_err);
        if (g_err<EPS) {
            printf(" TEST OK\n\n");
        } else {
            printf(" TEST FAILED\n\n");
            failure++;
        }
    }

/*--- Usage of MPI FFTW for Real-to-Complex One-dimensional Transforms ---*/

    if (PrR==0) printf("Usage of MPI FFTW for Real-to-Complex One-dimensional Transforms\n");
    plan=rfftwnd_mpi_create_plan(MPI_COMM_WORLD,1,&len,FFTW_FORWARD,FFTW_ESTIMATE);
    if (PrR==0) printf("     CreatePlan for Forward....DONE\n");

    rfftwnd_mpi_local_sizes(plan,&nx,&start_x,&nx_out,&start_x_out,&size);
    if (PrR==0) printf("     LocalSizes................DONE\n");
    local=(fftw_real*)fftw_malloc(size*sizeof(fftw_complex));
    work=(fftw_real*)fftw_malloc(size*sizeof(fftw_complex));
    preal=(fftw_real*)local;

    srand(PrR*100);

    for (i=0;i<nx;i++)
        preal[i]=(fftw_real)rand()/RAND_MAX;

    rfftwnd_mpi(plan,1,local,work,FFTW_NORMAL_ORDER);
    if (PrR==0) printf("     Forward FFTW..............DONE\n");

    rfftwnd_mpi_destroy_plan(plan);
    if (PrR==0) printf("     Destroy plan for Forward..DONE\n");

    plan=rfftwnd_mpi_create_plan(MPI_COMM_WORLD,1,&len,FFTW_BACKWARD,FFTW_ESTIMATE);
    if (PrR==0) printf("     CreatePlan for Backward...DONE\n");

    rfftwnd_mpi(plan,1,local,work,FFTW_NORMAL_ORDER);
    scale=1.0/len;
    if (PrR==0) printf("     Backward FFTW.............DONE\n");

    for (i=0;i<nx;i++)
        preal[i]*=scale;

    rfftwnd_mpi_destroy_plan(plan);
    if (PrR==0) printf("     Destroy plan for Backward.DONE\n");
    srand(PrR*100);
    err=0.0;
    for (i=0;i<nx;i++) {
        t=fabs(preal[i]-(fftw_real)rand()/RAND_MAX);
        if (t>err) err=t;
    }
    fftw_free(local);
    fftw_free(work);
#ifdef FFTW_ENABLE_FLOAT
    MPI_Reduce(&err,&g_err,1,MPI_FLOAT,MPI_MAX,0,MPI_COMM_WORLD);
#else
    MPI_Reduce(&err,&g_err,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
#endif
    MPI_Finalize();

    if (PrR==0) {
        printf("     Error=%e\n",g_err);
        if (g_err<EPS) {
            printf(" TEST PASSED\n");
        } else {
            printf(" TEST FAILED\n");
            return 1;
        }
    }

    if (failure)
        return 1;

    if (PrR==0) printf(" END OF TEST\n");

    return 0;
}
