/*****************************************************************************
!                             INTEL CONFIDENTIAL
! Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
! The source code contained  or  described herein and all documents related to
! the source code ("Material") are owned by Intel Corporation or its suppliers
! or licensors.  Title to the  Material remains with  Intel Corporation or its
! suppliers and licensors. The Material contains trade secrets and proprietary
! and  confidential  information of  Intel or its suppliers and licensors. The
! Material  is  protected  by  worldwide  copyright  and trade secret laws and
! treaty  provisions. No part of the Material may be used, copied, reproduced,
! modified, published, uploaded, posted, transmitted, distributed or disclosed
! in any way without Intel's prior express written permission.
! No license  under any  patent, copyright, trade secret or other intellectual
! property right is granted to or conferred upon you by disclosure or delivery
! of the Materials,  either expressly, by implication, inducement, estoppel or
! otherwise.  Any  license  under  such  intellectual property  rights must be
! express and approved by Intel in writing.
!
!*****************************************************************************
! Content:
!       MKL DFTI implementation through FFTW interface (via wrappers) example
!       program (C-interface)
!
! Real-to-complex and complex-to-real 1D transform for double precision data
! inplace CCE packed format for complex conjugate-symmetric data
!
! Configuration parameters:
!           DFTI_FORWARD_DOMAIN = DFTI_REAL       (obligatory)
!           DFTI_PRECISION      = DFTI_DOUBLE     (obligatory)
!           DFTI_DIMENSION      = 1               (obligatory)
!           DFTI_LENGTHS        = n               (obligatory)
!           DFTI_PACKED_FORMAT  = DFTI_CCE_FORMAT (default)
!           DFTI_PLACEMENT      = DFTI_INPLACE    (default)
!           DFTI_FORWARD_SCALE  = 1.0             (default)
!           DFTI_BACKWARD_SCALE = 1.0/n           (default=1.0)
!           DFTI_REAL_STORAGE   = DFTI_REAL_REAL  (default)
!           DFTI_CONJUGATE_EVEN_STORAGE = DFTI_COMPLEX_COMPLEX
!                                                 (default=DFTI_COMPLEX_REAL)
!
! Other default configuration parameters are in the mkl_dfti.h interface file
!****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "fftw3.h"
#include "mkl_dfti_examples.h"

int main(void)  /* REAL_1D_CCE_DOUBLE_EX10 */
{
    /*
    **  DFT input parameters
    */
    fftw_plan my_plan;
    double*  x_in;
    long     n = 7;
    int      rank = 1;

    double*  x_exp;
    double   Scale;
    double   maxerr;
    double   eps = DOUBLE_EPS;
    int      ni;
    long     nn;

    if (LEGEND_PRINT) {
        printf("\n\n REAL_1D_CCE_DOUBLE_EX10\n");
        printf(" Forward-Backward 1D real transform for double precision data\n\n");
        printf(" Configuration parameters:\n\n");
        printf(" DFTI_FORWARD_DOMAIN = DFTI_REAL\n");
        printf(" DFTI_PRECISION      = DFTI_DOUBLE\n");
        printf(" DFTI_DIMENSION      = 1\n");
        printf(" DFTI_LENGTHS        = %d\n", n);
        printf(" DFTI_PACKED_FORMAT  = DFTI_CCE_FORMAT\n");
        printf(" DFTI_PLACEMENT      = DFTI_INPLACE\n");
        printf(" DFTI_FORWARD_SCALE  = 1.0\n");
        printf(" DFTI_BACKWARD_SCALE = 1.0/n\n\n");
    }

    /*
    **  Allocate arrays for input and expected data
    */
    nn = 2 * (n/2+1);
    x_in  = (double*)fftw_malloc(nn*sizeof(double));
    x_exp = (double*)fftw_malloc(nn*sizeof(double));

    /*
    **  Initialize x_in and copy to expected x_exp
    */
    zero_init_d(x_in, nn);
    init_real_vectors_d(x_in, nn);
    cblas_dcopy(nn, x_in, 1, x_exp, 1);

    ni = (int)n;

    /*
    **  Create FFTW plan for 1D double precision forward transform
    */
    my_plan = fftw_plan_dft_r2c(rank, &ni, x_in, (fftw_complex*)x_in, FFTW_ESTIMATE);

    /*
    **  Compute DFT
    */
    fftw_execute(my_plan);

    /*
    **  Destroy FFTW plan
    */
    fftw_destroy_plan(my_plan);

    /*
    **  Set Scale number for Backward transform
    */
    Scale = 1.0/(double)n;

    /*
    **  Create FFTW plan for 1D double precision backward transform
    */
    my_plan = fftw_plan_dft_c2r(rank, &ni, (fftw_complex*)x_in, x_in, FFTW_ESTIMATE);

    /*
    **  Compute DFT
    */
    fftw_execute(my_plan);

    /*
    **  Destroy FFTW plan
    */
    fftw_destroy_plan(my_plan);

    /*
    **  Result scaling
    */
    scaling_dr(x_in, Scale, n);

    /*
    **  Check result
    */
    maxerr = check_result_d(x_in, x_exp, n);

    /*
    **  Free arrays for input and expected data
    */
    fftw_free(x_in);
    fftw_free(x_exp);

    /*
    **  TEST result 
    */
    if (ACCURACY_PRINT)
        printf("\n Accuracy = %g\n\n", maxerr);

    if (maxerr < eps) {
        printf(" TEST PASSED\n");
    } else {
        printf(" TEST FAILED\n");
        return 1;
    }

    printf(" END OF TEST\n");

    return 0;
}
