!*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!*******************************************************************************
!  Content: Intel MKL RCI (P)CG Fortran-77 example
!
!*******************************************************************************

!---------------------------------------------------------------------------
! Example program for solving symmetric positive definite system of
! equations A*x =f where A is a symmetric tridiagonal matrix of order N whose
! nonzero elements are defined as follows
!
!         a(i, i) = 1,     i=1, N;
!         a(i, i+1)= 1/2,    i=1, N-1;
!         a(i+1, i) = 1/2,   i=1, N-1;
!
! The system is solved with the help of the conjugate gradient method where
! the Symmetric Successive Over Relaxation (SSOR) stationary iterative solver
! with the given number of iterations is used as the preconditioner. The relaxation
! parameter is set to 1/2, the number of iterations for the SSOR preconditioner
! is equal to 20. For simplicity, we don't check the convergence while solving
! equations with the help of SSOR. Let us recall that the scheme of SSOR is
! the following:
!
!         (D+w*U^{T}) x_{k+1} = (-w*U+(1-w)*D)*x_{k}+w*f
!
! where w is the relaxation parameter, f is the right hand side, D is the diagonal
! of A, U its strict upper part so that
!
!                  A   =  U^{T} + D + U
!
! The compressed sparse row format is used for storing nonzeros of A and
! since D is the identity matrix, we don't need to store diagonal elements. So
! the code given below only uses the sparse representation of U. The Intel
! MKL Sparse BLAS is designed so that it allows the user to perform large
! variety of operations with one sparse representation setting appropriate
! values of the descriptor array (see MKL User Guide for the further details).
!
! Full case: full functionality of RCI (P)CG is used.
!---------------------------------------------------------------------------
      PROGRAM rci_pcg_f77_test_ssor
      IMPLICIT NONE
      INCLUDE 'mkl_rci.fi'
!---------------------------------------------------------------------------
! Define arrays for the upper triangle of the coefficient matrix and
! preconditioner as well as an array for rhs vector
! Compressed sparse row storage is used for sparse representation
!---------------------------------------------------------------------------
      INTEGER N, RCI_request, itercount, expected_itercount, i
      PARAMETER (N=100)
      PARAMETER (expected_itercount=19)
      DOUBLE PRECISION  rhs(N)
      INTEGER IA(N+1)
      INTEGER JA(N-1)
      DOUBLE PRECISION A(N-1), A1(N-1)
!---------------------------------------------------------------------------
! Allocate storage for the solver ?par and the initial solution vector
!---------------------------------------------------------------------------
      INTEGER length
      PARAMETER (length=128)
      INTEGER ipar(length)
      DOUBLE PRECISION dpar(length),TMP(N,4)
!---------------------------------------------------------------------------
! Some additional variables to use with the RCI (P)CG solver
! OMEGA is the relaxation parameter, NITER_SSOR is the maximum number of
! iterations for the SSOR preconditioner
!---------------------------------------------------------------------------
      DOUBLE PRECISION solution(N)
      DOUBLE PRECISION expected_sol(N)
      DOUBLE PRECISION OMEGA, ONE, ZERO
      DATA  OMEGA/0.5D0/, ONE/1.D0/, ZERO/0.D0/
      DOUBLE PRECISION DNRM2, Euclidean_norm, temp(N)
      INTEGER   NITER_SSOR
      DATA      NITER_SSOR/20/
      CHARACTER MATDES(6)

      EXTERNAL DNRM2
!---------------------------------------------------------------------------
! Initialize the coefficient matrix and expected solution
!---------------------------------------------------------------------------
      DO I=1, N
         expected_sol(I)=1.D0
      ENDDO
      DO I=1, N-1
         JA(I)=I+1
         IA(I)=I
         A(I)=0.5D0
         A1(I)=OMEGA*A(I)
      ENDDO
      IA(N)=N
      IA(N+1)=IA(N)
      MATDES(1)='S'
      MATDES(2)='U'
      MATDES(3)='U'
      MATDES(4)='F'
!---------------------------------------------------------------------------
! Initialize vectors rhs, TEMP, and TMP(:,2) with zeros as MKL_DCSRMV
! routine does not set NAN to zero. Thus, if any of the values in the
! vectors above accidentally happens to be NAN, the example will fail
! to complete.
! Initialize the right hand side through matrix-vector product
!---------------------------------------------------------------------------
      DO I=1, N
         rhs(I)=ZERO
         TEMP(I)=ZERO
         TMP(I,2)=ZERO
      ENDDO
      CALL MKL_DCSRMV('N', N, N, ONE, MATDES,
     &            A, JA, IA, IA(2), expected_sol, ZERO, rhs)

!---------------------------------------------------------------------------
! Initialize the initial guess
!---------------------------------------------------------------------------
         DO I=1, N
           solution(I)=ZERO
         ENDDO
!---------------------------------------------------------------------------
! Initialize the solver
!---------------------------------------------------------------------------
      CALL dcg_init(N,solution,rhs,RCI_request,ipar,dpar,TMP)
      IF (RCI_request .NE. 0) GOTO 999
!---------------------------------------------------------------------------
! Set the desired parameters:
! INTEGER parameters:
! set the maximal number of iterations to 100
! LOGICAL parameters:
! run the Preconditioned version of RCI (P)CG with preconditioner C_inverse
! DOUBLE PRECISION parameters
! -
!---------------------------------------------------------------------------
      ipar(5)=100
      ipar(11)=1
!---------------------------------------------------------------------------
! Check the correctness and consistency of the newly set parameters
!---------------------------------------------------------------------------
      CALL dcg_check(N,solution,rhs,RCI_request,ipar,dpar,TMP)
      IF (RCI_request .NE. 0 ) GOTO 999
!---------------------------------------------------------------------------
! Compute the solution by RCI (P)CG solver
! Reverse Communications starts here
!---------------------------------------------------------------------------
1     CALL dcg(N,solution,rhs,RCI_request,ipar,dpar,TMP)
!---------------------------------------------------------------------------
! If RCI_request=0, then the solution was found according to the requested
! stopping tests. In this case, this means that it was found after 100
! iterations.
!---------------------------------------------------------------------------
      IF (RCI_request .EQ. 0) THEN
          GOTO 700
!---------------------------------------------------------------------------
! If RCI_request=1, then compute the vector A*TMP(:,1)
! and put the result in vector TMP(:,2)
!---------------------------------------------------------------------------
      ELSEIF (RCI_request .EQ. 1) THEN
          MATDES(1)='S'
          CALL MKL_DCSRMV('N', N, N, ONE, MATDES,
     &            A, JA, IA, IA(2), TMP, ZERO, TMP(1,2))
        GOTO 1
!---------------------------------------------------------------------------
! If RCI_request=2, then do the user-defined stopping test: compute the
! Euclidean norm of the actual residual using MKL routines and check if
! it is less than 1.D-8
!---------------------------------------------------------------------------
      ELSEIF (RCI_request .EQ. 2) THEN
          MATDES(1)='S'
          CALL MKL_DCSRMV('N', N, N, ONE, MATDES,
     &            A, JA, IA, IA(2), SOLUTION, ZERO, TEMP)
         CALL DAXPY(N,-1.D0,rhs,1,TEMP,1)
         Euclidean_norm = DNRM2(N,TEMP,1)
         IF (Euclidean_norm .GT. 1.D-6) THEN
!---------------------------------------------------------------------------
! The solution has not been found yet according to the user-defined stopping
! test. Continue RCI (P)CG iterations.
!---------------------------------------------------------------------------
          GOTO 1
        ELSE
!---------------------------------------------------------------------------
! The solution has been found according to the user-defined stopping test
!---------------------------------------------------------------------------
          GOTO 700
        END IF
!---------------------------------------------------------------------------
! If RCI_request=3, then  apply the simplest SSOR preconditioning
! on vector TMP(:,3) and put the result in vector TMP(:,4)
!---------------------------------------------------------------------------
      ELSEIF (RCI_request .EQ. 3) THEN
         CALL DCOPY(N, TMP(1,3), 1, TMP(1, 4), 1)
         MATDES(1)='T'
         DO I=1, NITER_SSOR
            CALL DCOPY(N, TMP(1, 3), 1, TEMP, 1)
            MATDES(3)='N'
            CALL MKL_DCSRMV('N', N, N, -ONE, MATDES,
     &             A1, JA, IA, IA(2), TMP(1, 4), OMEGA, TEMP)
            CALL DAXPY(N, ONE-OMEGA, TMP(1,4), 1, TEMP, 1)
            MATDES(3)='U'
            CALL MKL_DCSRSV('T', N, ONE, MATDES,
     &             A1, JA, IA, IA(2), TEMP, TMP(1,4))
         ENDDO
        GOTO 1
      ELSE
!---------------------------------------------------------------------------
! If RCI_request=anything else, then dcg subroutine failed
! to compute the solution vector: solution(N)
!---------------------------------------------------------------------------
        GOTO 999
      ENDIF
!---------------------------------------------------------------------------
! Reverse Communication ends here
! Get the current iteration number
!---------------------------------------------------------------------------
700   CALL dcg_get(N,solution,rhs,RCI_request,ipar,dpar,TMP,
     &                    itercount)
!---------------------------------------------------------------------------
! Print solution vector: solution(N) and number of iterations: itercount
!---------------------------------------------------------------------------
      WRITE(*, *) ' The system has been solved '
      WRITE(*, *) ' The following solution obtained '
      WRITE(*,800) (solution(i),i =1,N)
      WRITE(*, *) ' expected solution '
      WRITE(*,800)(expected_sol(i),i =1,N)
800   FORMAT(4(F10.3))
      WRITE(*,900)(itercount)
900   FORMAT(' Number of iterations: ', I5)
      DO I=1,N
         expected_sol(I)=expected_sol(I)-solution(I)
      ENDDO

      Euclidean_norm=DNRM2(N,expected_sol,1)

!---------------------------------------------------------------------------
! Release internal MKL memory that might be used for computations
! NOTE: It is important to call the routine below to avoid memory leaks
! unless you disable MKL Memory Manager
!---------------------------------------------------------------------------
      CALL MKL_FREEBUFFERS

      IF (itercount.EQ.expected_itercount .AND.
     1                                   Euclidean_norm.LE.1.0D-4) THEN
         WRITE( *,'(A,A)') 'This example has successfully PASSED',
     1 ' through all steps of computation!'
         STOP 0
      ELSE
         WRITE( *,'(A,A,A,I5,A,A,A,E12.5,A)') 'This example may have',
     1 ' FAILED as either the number of iterations differs from the',
     2 ' expected number of iterations ',expected_itercount,' or the',
     3 ' computed solution differs much from the expected solution (',
     4 'Euclidean norm is ',Euclidean_norm,'), or both.'
         STOP 1
      ENDIF
!---------------------------------------------------------------------------
! Release internal MKL memory that might be used for computations
! NOTE: It is important to call the routine below to avoid memory leaks
! unless you disable MKL Memory Manager
!---------------------------------------------------------------------------
999   WRITE( *,'(A,A)') 'This example FAILED as the solver has',
     1 ' returned the ERROR code', RCI_request
      CALL MKL_FREEBUFFERS
      STOP 1

      END
