/*
********************************************************************************
*                              INTEL CONFIDENTIAL
*   Copyright(C) 2004-2010 Intel Corporation. All Rights Reserved.
*   The source code contained  or  described herein and all documents related to
*   the source code ("Material") are owned by Intel Corporation or its suppliers
*   or licensors.  Title to the  Material remains with  Intel Corporation or its
*   suppliers and licensors. The Material contains trade secrets and proprietary
*   and  confidential  information of  Intel or its suppliers and licensors. The
*   Material  is  protected  by  worldwide  copyright  and trade secret laws and
*   treaty  provisions. No part of the Material may be used, copied, reproduced,
*   modified, published, uploaded, posted, transmitted, distributed or disclosed
*   in any way without Intel's prior express written permission.
*   No license  under any  patent, copyright, trade secret or other intellectual
*   property right is granted to or conferred upon you by disclosure or delivery
*   of the Materials,  either expressly, by implication, inducement, estoppel or
*   otherwise.  Any  license  under  such  intellectual property  rights must be
*   express and approved by Intel in writing.
*
********************************************************************************
*   Content : MKL PARDISO C example
*
********************************************************************************
/* -------------------------------------------------------------------- */
/*      Example program to show the use of the "PARDISO" routine        */
/*      for complex unsymmetric linear systems                          */
/* -------------------------------------------------------------------- */
/*      This program can be downloaded from the following site:         */
/*      www.pardiso-project.org                                         */
/*                                                                      */
/*  (C) Olaf Schenk, Department of Computer Science,                    */
/*      University of Basel, Switzerland.                               */
/*      Email: olaf.schenk@unibas.ch                                    */
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mkl_pardiso.h"
#include "mkl_types.h"

typedef struct {	
       double re;	
       double i;
} doublecomplex;

MKL_INT main( void )
{
    /* Matrix data. */

    MKL_INT    n = 8;
    MKL_INT    ia[ 9] = { 1, 5, 8, 10, 12, 13, 16, 18, 21 };
    MKL_INT    ja[20] = { 1,    3,       6, 7,
                             2, 3,    5,
                                3,             8,
                                   4,       7,
                             2,
                                3,       6,    8,
                             2,             7,
                                3,          7, 8 };

   doublecomplex  a[20] = { a[0 ].re =  7.0, a[0 ].i = 1.0,
			    a[1 ].re =  1.0, a[1 ].i = 1.0,
			    a[2 ].re =  2.0, a[2 ].i = 1.0,
			    a[3 ].re =  7.0, a[3 ].i = 1.0,
			    a[4 ].re = -4.0, a[4 ].i = 0.0,
			    a[5 ].re =  8.0, a[5 ].i = 1.0,
			    a[6 ].re =  2.0, a[6 ].i = 1.0,
			    a[7 ].re =  1.0, a[7 ].i = 1.0,
			    a[8 ].re =  5.0, a[8 ].i = 1.0,
			    a[9 ].re =  7.0, a[9 ].i = 0.0,
			    a[10].re =  9.0, a[10].i = 1.0,
			    a[11].re = -4.0, a[11].i = 1.0,
			    a[12].re =  7.0, a[12].i = 1.0,
			    a[13].re =  3.0, a[13].i = 1.0,
			    a[14].re =  8.0, a[14].i = 0.0,
			    a[15].re =  1.0, a[15].i = 1.0,
			    a[16].re = 11.0, a[16].i = 1.0,
			    a[17].re = -3.0, a[17].i = 1.0,
			    a[18].re =  2.0, a[18].i = 1.0,
			    a[19].re =  5.0, a[19].i = 0.0};

    MKL_INT      mtype = 13;        /* Real complex unsymmetric matrix */


    /* RHS and solution vectors. */
    doublecomplex   b[8], x[8];
    MKL_INT      nrhs = 1;          /* Number of right hand sides. */

    /* Internal solver memory pointer pt,                  */
    /* 32-bit: int pt[64]; 64-bit: long int pt[64]         */
    /* or void *pt[64] should be OK on both architectures  */
    void    *pt[64];

    /* Pardiso control parameters. */
    MKL_INT      iparm[64];
    MKL_INT      maxfct, mnum, phase, error, msglvl;

    /* Auxiliary variables. */
    MKL_INT      i;

    doublecomplex   ddum;        /* Double dummy */
    MKL_INT	      idum;              /* Integer dummy. */

/* -------------------------------------------------------------------- */
/* .. Setup Pardiso control parameters. */
/* -------------------------------------------------------------------- */
    for (i = 0; i < 64; i++) {
	iparm[i] = 0;
    }
    iparm[0] = 1; /* No solver default */
    iparm[1] = 2; /* Fill-in reordering from METIS */
    /* Numbers of processors, value of OMP_NUM_THREADS */
    iparm[2] = 1;
    iparm[3] = 0; /* No iterative-direct algorithm */
    iparm[4] = 0; /* No user fill-in reducing permutation */
    iparm[5] = 0; /* Write solution into x */
    iparm[6] = 0; /* Not in use */
    iparm[7] = 2; /* Max numbers of iterative refinement steps */
    iparm[8] = 0; /* Not in use */
    iparm[9] = 13; /* Perturb the pivot elements with 1E-13 */
    iparm[10] = 1; /* Use nonsymmetric permutation and scaling MPS */
    iparm[11] = 0; /* Not in use */
    iparm[12] = 1; /* Maximum weighted matching algorithm is switched-on (default for non-symmetric) */
    iparm[13] = 0; /* Output: Number of perturbed pivots */
    iparm[14] = 0; /* Not in use */
    iparm[15] = 0; /* Not in use */
    iparm[16] = 0; /* Not in use */
    iparm[17] = -1; /* Output: Number of nonzeros in the factor LU */
    iparm[18] = -1; /* Output: Mflops for LU factorization */
    iparm[19] = 0; /* Output: Numbers of CG Iterations */

    maxfct = 1;         /* Maximum number of numerical factorizations.  */
    mnum   = 1;         /* Which factorization to use. */

    msglvl = 1;         /* Print statistical information  */
    error  = 0;         /* Initialize error flag */

/* -------------------------------------------------------------------- */
/* .. Initialize the internal solver memory pointer. This is only */
/* necessary for the FIRST call of the PARDISO solver. */
/* -------------------------------------------------------------------- */
    for (i = 0; i < 64; i++) {
	pt[i] = 0;
    }

/* -------------------------------------------------------------------- */
/* ..  Reordering and Symbolic Factorization.  This step also allocates */
/*     all memory that is necessary for the factorization.              */
/* -------------------------------------------------------------------- */
    phase = 11;

    PARDISO (pt, &maxfct, &mnum, &mtype, &phase,
	    &n, a, ia, ja, &idum, &nrhs,
	    iparm, &msglvl, &ddum, &ddum, &error);

    if (error != 0) {
        printf("\nERROR during symbolic factorization: %d", error);
        exit(1);
    }
    printf("\nReordering completed ... ");
    printf("\nNumber of nonzeros in factors  = %d", iparm[17]);
    printf("\nNumber of factorization MFLOPS = %d", iparm[18]);

/* -------------------------------------------------------------------- */
/* ..  Numerical factorization.                                         */
/* -------------------------------------------------------------------- */
    phase = 22;

    PARDISO (pt, &maxfct, &mnum, &mtype, &phase,
	    &n, a, ia, ja, &idum, &nrhs,
	    iparm, &msglvl, &ddum, &ddum, &error);

    if (error != 0) {
        printf("\nERROR during numerical factorization: %d", error);
        exit(2);
    }
    printf("\nFactorization completed ...\n ");

/* -------------------------------------------------------------------- */
/* ..  Back substitution and iterative refinement.                      */
/* -------------------------------------------------------------------- */
    phase = 33;

    iparm[7] = 1;       /* Max numbers of iterative refinement steps. */

    /* Set right hand side to one. */
    for (i = 0; i < n; i++)
    {
		b[i].re = 1;
		b[i].i	= 1;
    }
    PARDISO (pt, &maxfct, &mnum, &mtype, &phase,
	    &n, a, ia, ja, &idum, &nrhs,
	    iparm, &msglvl, b, x, &error);

    if (error != 0) {
        printf("\nERROR during solution: %d", error);
        exit(3);
    }

    printf("\nSolve completed ... ");
    printf("\nThe solution of the system is: ");
    for (i = 0; i < n; i++) {
        printf("\n x [%d] = % f % f", i, x[i].re, x[i].i);
    }
    printf ("\n");
/* -------------------------------------------------------------------- */
/* ..  Termination and release of memory.                               */
/* -------------------------------------------------------------------- */
    phase = -1;                 /* Release internal memory. */

    PARDISO (pt, &maxfct, &mnum, &mtype, &phase,
	    &n, &ddum, ia, ja, &idum, &nrhs,
	    iparm, &msglvl, &ddum, &ddum, &error);

    return 0;
}
