!*******************************************************************************
!                              INTEL CONFIDENTIAL
!   Copyright(C) 2005-2010 Intel Corporation. All Rights Reserved.
!   The source code contained  or  described herein and all documents related to
!   the source code ("Material") are owned by Intel Corporation or its suppliers
!   or licensors.  Title to the  Material remains with  Intel Corporation or its
!   suppliers and licensors. The Material contains trade secrets and proprietary
!   and  confidential  information of  Intel or its suppliers and licensors. The
!   Material  is  protected  by  worldwide  copyright  and trade secret laws and
!   treaty  provisions. No part of the Material may be used, copied, reproduced,
!   modified, published, uploaded, posted, transmitted, distributed or disclosed
!   in any way without Intel's prior express written permission.
!   No license  under any  patent, copyright, trade secret or other intellectual
!   property right is granted to or conferred upon you by disclosure or delivery
!   of the Materials,  either expressly, by implication, inducement, estoppel or
!   otherwise.  Any  license  under  such  intellectual property  rights must be
!   express and approved by Intel in writing.
!
!*******************************************************************************
!   Content : MKL Sparse BLAS Fortran-77 example
!
!*******************************************************************************
!
! Example program for using MKL Sparse BLAS Level 2 and 3
! for matrices represented in the coordinate storage scheme.
! The following Sparse  Blas routines are used in the example:
!          MKL_ZCOOSM  MKL_ZCOOSV  MKL_ZCOOMM  MKL_ZCOOMV
!          MKL_ZCOOGEMV    MKL_ZCOOSYMV  MKL_ZCOOTRSV.   
!
! Consider the matrix A 
!
!                 |   1,1     -1,1   -3,1  0     0   |
!                 |  -2,1      5,1    0    0     0   |
!   A    =        |   0        0      4,1  6,1   4,1 |,
!                 |  -4,1      0      2,1  7,1   0   |
!                 |   0        8,1    0    0    -5,1 |
!
!  
! decomposed as 
!
!                      A = L + D + U,      
!
!  where L is the strict  lower triangle of A, U is the strictly  upper triangle
!  of A, D is the main diagonal. Namely 
!   
!        |   0    0   0    0     0   |       |  0   -1,1 -3,1  0   0   | 
!        |  -2,1  0   0    0     0   |       |  0    0    0    0   0   |
!   L  = |   0    0   0    0     0   |,  U=  |  0    0    0    6,1 4,1 |
!        |  -4,1  0   2,1  0     0   |       |  0    0    0    0   0   |
!        |   0    8,1 0    0     0   |       |  0    0    0    0   0   |
!
!
!           |   1,1  0    0    0    0   |
!           |   0    5,1  0    0    0   |
!   D    =  |   0    0    4,1  0    0   |.
!           |   0    0    0    7,1  0   |
!           |   0    0    0    0   -5,1 |
! 
!  The matrix A given above is represented in the coordinate storage scheme with the help of three 
!  arrays of length nnz=13 (see Appendix 'Sparse Storage Formats for Sparse Blas level 2-3'
!
!          values = (1,1 -1,1 -3,1 -2,1 5,1 4,1 6,1 4,1 -4,1 2,1 7,1 8,1 -5,1) 
!          rows=    (1  1  1  2 2 3 3 3  4 4 4 5 5 ) 
!          columns = (1 2  3  1 2 3 4 5  1 3 4 2 5) 
!
!  In what follows the symbol ' means taking of transposed. 
!
!  The test performs the following operations : 
!    
!       1. The code computes (L+D)'*S = F using MKL_ZCOOMM where S is a known 5 by 2  
!          matrix and then the code solves the system (L+D)'*X = F with the help of MKL_ZCOOSM. 
!          It's evident that X should be equal to S.
!
!       2. The code computes (U+I)'*S = F using MKL_ZCOOMV where S is a vector
!          and then the code calls MKL_ZCOOTRSV solves the system (U+I)'*X = F with the single right 
!          hand side. It's evident that X should be equal to S.
!
!       3. The code computes D*S = F using MKL_ZCOOMV where S is a vector
!          and then the code solves the system D*X = F with the single right hand side. 
!          It's evident that X should be equal to S.
!
!       4. The next step is the computation (U-U') S = F using MKL_ZCOOMV where S is 
!          a vector. It is easy to see that U-U' is a skew-symmetric matrix.
!
!       5. The next step is the computation (L+D+L') S = F using MKL_ZCOOSYMV where S is 
!          a vector. It is easy to see that L+D+L' is a symmetric matrix.
!
!       6. The next step is the computation A'* S = F using MKL_ZCOOGEMV where S is 
!          a vector.
!
! The code given below uses only one sparse representation for the all operations.
!*******************************************************************************
!     Definition arrays for sparse representation of  the matrix A in 
!     the coordinate format: 
!******************************************************************************* 
          integer  m, nnz
          parameter( m = 5,  nnz=13)
          complex*16 values(nnz)
          integer  rows(nnz), columns(nnz)
          data values/(1.0d0, 1.0d0), (-1.0d0, 1.0d0), 
     &     (-3.0d0, 1.0d0), (-2.0d0, 1.0d0), (5.0d0, 1.0d0),
     &     (4.0d0, 1.0d0), (6.0d0, 1.0d0), (4.0d0, 1.0d0), 
     &     (-4.0d0, 1.0d0), (2.0d0, 1.0d0), (7.0d0, 1.0d0),
     &     (8.0d0, 1.0d0), (-5.0d0, 1.0d0)/ 
          data rows/1, 1, 1, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5/
          data columns/1, 2, 3, 1, 2, 3, 4, 5, 1, 3, 4, 2, 5/
!*******************************************************************************
!    Declaration of local variables : 
!*******************************************************************************
          integer n
          parameter (n=2)
          complex*16 rhs(m, n), sol(m, n), temp(m, n) 
          data sol/(1.0d0, 1.0d0), (1.0d0, 1.0d0), (1.0d0, 1.0d0),
     &      (1.0d0, 1.0d0), (1.0d0, 1.0d0), (5.0d0, 1.0d0),
     &      (4.0d0, 1.0d0), (3.0d0, 1.0d0), (2.0d0, 1.0d0), 
     &      (1.0d0, 1.0d0)/
          complex*16 alpha, beta
          data alpha/(1.0d0, 0.0d0)/, beta/(0.0d0, 0.0d0)/
          integer i, j
          print*
          print*, ' EXAMPLE PROGRAM FOR COORDINATE FORMAT ROUTINES ' 

!*******************************************************************************
!    Task 1.    Obtain matrix-matrix multiply (L+D)' *sol --> rhs
!    and solve triangular system   (L+D)' *temp = rhs with multiple right hand sides
!    Array temp must be equal to the array sol 
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOMM '
          print*, '     WITH TRIANGULAR MATRIX  '
          print 101, m, n
          print 102, alpha, beta
          print 103, 't'
          print*, ' Input matrix '
          print 104, ((sol(i,j),j=1,n),i=1,m)
         
           call mkl_zcoomm('t', m, n, m, alpha, 'tln',
     &           values, rows, columns, nnz, sol, m,  beta, rhs,  m)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOMM '
          print*, '     WITH TRIANGULAR MATRIX  '
          print 104, ((rhs(i,j),j=1,n),i=1,m)
          print 100
          print*, ' Solve triangular system with obtained '
          print*, ' right hand side  '
          call mkl_zcoosm('t', m, n, alpha, 'tln',
     &           values, rows, columns, nnz, rhs, m, temp,  m)

          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOSM '
          print 104, ((temp(i,j),j=1,n),i=1,m)
          print 100
!*******************************************************************************
!    Task 2.    Obtain matrix-vector multiply (U+I)' *sol --> rhs
!    and solve triangular system   (U+I)' *temp = rhs with single right hand sides
!    Array temp must be equal to the array sol 
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH TRIANGULAR MATRIX  '
          print 102, alpha, beta
          print 103, 't'
          print*, ' Input vector '
          print 105, (sol(i,1),i=1,m)
         
           call mkl_zcoomv('t', m, m, alpha, 'tuu',
     &           values, rows, columns, nnz, sol, beta, rhs)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH TRIANGULAR MATRIX  '
          print 105, (rhs(i,1),i=1,m)
          print 100
          print*, ' Solve triangular system with obtained '
          print*, ' right hand side  '
          call mkl_zcootrsv('u', 't', 'u', m, 
     &           values, rows, columns, nnz, rhs, temp)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOTRSV '
          print*, '     WITH TRIANGULAR MATRIX  '
          print 105, (temp(i,1),i=1,m)
          print 100
!*******************************************************************************
!    Task 3.  Obtain matrix-vector multiply D *sol --> rhs
!    and solve triangular system   D *temp = rhs with single right hand side
!    Array temp must be equal to the array sol 
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH DIAGONAL MATRIX  '
          print 102, alpha, beta
          print 103, 't'
          print*, ' Input vector '
          print 105, (sol(i,2),i=1,m)
         
           call mkl_zcoomv('t', m, m, alpha, 'dun',
     &          values, rows, columns, nnz,  sol(1,2), beta, rhs)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH DIAGONAL MATRIX  '
          print 105, (rhs(i,1),i=1,m)
          print 100
          print*, ' Multiply by inverse matrix '
          print*, ' with the help of MKL_ZCOOSV '


          call mkl_zcoosv('t', m,  alpha, 'dun',
     &         values, rows, columns, nnz,  rhs, temp)

          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOSV '
          print*, '     WITH DIAGONAL MATRIX  '
          print 105, (temp(i,1),i=1,m)
          print 100

!*******************************************************************************
!    Task 4.  Obtain matrix-vector multiply (U -U')*sol --> rhs
!    Array temp must be equal to the array sol 
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH SKEW-SYMMETRIC MATRIX '
          print 102, alpha, beta
          print 103, 'n'
          print*, ' Input vector '
          print 105, (sol(i, 1),i=1,m)
         
           call mkl_zcoomv('n', m, m, alpha, 'au',
     &          values, rows, columns, nnz, sol, beta, rhs)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOMV '
          print*, '     WITH SKEW-SYMMETRIC MATRIX  '
          print 105, (rhs(i,1),i=1,m)
          print 100
!*******************************************************************************
!    Task 5.  Obtain matrix-vector multiply (L+D+L')*sol --> rhs whith the help of 
!    MKL_ZCOOSYMV
!   
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOSYMV '
          print*, '     WITH SYMMETRIC MATRIX  '
          print 102, alpha, beta
          print*, ' Input vector '
          print 105, (sol(i, 1),i=1,m)
         
           call mkl_zcoosymv('l', m,  values, rows, columns, nnz,
     &           sol, rhs)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOSYMV '
          print*, '     WITH SYMMETRIC MATRIX  '
          print 105, (rhs(i,1),i=1,m)
          print 100
!*******************************************************************************
!    Task 6. Obtain matrix-vector multiply A'*sol --> rhs whith the help 
!    of MKL_ZCOOGEMV
!   
!*******************************************************************************
          print*
          print*, '     INPUT DATA FOR MKL_ZCOOGEMV '
          print*, '     WITH GENERAL MATRIX  '
          print 102, alpha, beta
          print 103, 't'
          print*, ' Input vector '
          print 105, (sol(i, 1),i=1,m)
         
           call mkl_zcoogemv('t', m,  values, rows, columns, nnz,
     &           sol, rhs)
          print*
          print*, '     OUTPUT DATA FOR MKL_ZCOOGEMV '
          print*, '     WITH GENERAL MATRIX  '
          print 105, (rhs(i,1),i=1,m)
          print 100

 100      format('------------------------------------------------')
 101      format(7x,'M=',i1,'  N=',i1)
 102      format(7x,'ALPHA= ',f4.1,' BETA= ', f4.1)
 103      format(7x,'TRANS=',a1)
 104      format(2(f7.1, 3x))
 105      format(f4.1)
          stop 
          end
