/******************************************************************************
 *                            INTEL CONFIDENTIAL
 * Copyright(C) 2008-2010 Intel Corporation. All Rights Reserved.
 * The source code contained  or  described herein and all documents related to
 * the source code ("Material") are owned by Intel Corporation or its suppliers
 * or licensors.  Title to the  Material remains with  Intel Corporation or its
 * suppliers and licensors. The Material contains trade secrets and proprietary
 * and  confidential  information of  Intel or its suppliers and licensors. The
 * Material  is  protected  by  worldwide  copyright  and trade secret laws and
 * treaty  provisions. No part of the Material may be used, copied, reproduced,
 * modified, published, uploaded, posted, transmitted, distributed or disclosed
 * in any way without Intel's prior express written permission.
 * No license  under any  patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or delivery
 * of the Materials,  either expressly, by implication, inducement, estoppel or
 * otherwise.  Any  license  under  such  intellectual property  rights must be
 * express and approved by Intel in writing.
 *
 ******************************************************************************
 *
 * fftw_plan_guru_dft_r2c - FFTW3 wrapper to MKL.
 *
 ******************************************************************************
 */

#include "fftw3_mkl.h"

/* r2c is MKL DFTI real forward */
static void execute_fi(fftw_mkl_plan p);
static void execute_fo(fftw_mkl_plan p);

fftw_plan
fftw_plan_guru64_dft_r2c(int rank, const fftw_iodim64 *dims, int howmany_rank,
                         const fftw_iodim64 *howmany_dims, double *in,
                         fftw_complex *out, unsigned flags)
{
    fftw_mkl_plan mkl_plan;
    MKL_LONG periods[MKL_MAXRANK];
    MKL_LONG istrides[MKL_MAXRANK + 1];
    MKL_LONG ostrides[MKL_MAXRANK + 1];
    MKL_LONG s = 0;                    /* status */
    int i;

    UNUSED(flags);

    if (rank > MKL_MAXRANK || howmany_rank > MKL_ONE)
        return NULL;

    mkl_plan = fftw3_mkl.new_plan();
    if (!mkl_plan)
        return NULL;

    istrides[0] = 0;
    ostrides[0] = 0;

    for (i = 0; i < rank; ++i)
    {
        periods[i] = (MKL_LONG)dims[i].n;
        istrides[i + 1] = (MKL_LONG)dims[i].is;
        ostrides[i + 1] = (MKL_LONG)dims[i].os;

        /* check if MKL_LONG is sufficient to hold dims */
        if (periods[i] != dims[i].n)
            goto broken;
        if (istrides[i + 1] != dims[i].is)
            goto broken;
        if (ostrides[i + 1] != dims[i].os)
            goto broken;
    }
    if (rank == 1)
        s = DftiCreateDescriptor(&mkl_plan->desc, DFTI_DOUBLE, DFTI_REAL,
                                 (MKL_LONG)rank, periods[0]);
    else
        s = DftiCreateDescriptor(&mkl_plan->desc, DFTI_DOUBLE, DFTI_REAL,
                                 (MKL_LONG)rank, periods);
    if (BAD(s))
        goto broken;

    s = DftiSetValue(mkl_plan->desc, DFTI_CONJUGATE_EVEN_STORAGE,
                     DFTI_COMPLEX_COMPLEX);
    if (BAD(s))
        goto broken;

    s = DftiSetValue(mkl_plan->desc, DFTI_INPUT_STRIDES, istrides);
    if (BAD(s))
        goto broken;

    s = DftiSetValue(mkl_plan->desc, DFTI_OUTPUT_STRIDES, ostrides);
    if (BAD(s))
        goto broken;

    if ((void *)in == (void *)out)
    {
        mkl_plan->io[0] = in;
        mkl_plan->execute = execute_fi;
    }
    else
    {
        mkl_plan->io[0] = in;
        mkl_plan->io[1] = out;
        mkl_plan->execute = execute_fo;
        s = DftiSetValue(mkl_plan->desc, DFTI_PLACEMENT, DFTI_NOT_INPLACE);
        if (BAD(s))
            goto broken;
    }

    if (howmany_rank == 1)
    {
        MKL_LONG howmany = (MKL_LONG)howmany_dims[0].n;
        MKL_LONG idistance = (MKL_LONG)howmany_dims[0].is;
        MKL_LONG odistance = (MKL_LONG)howmany_dims[0].os;

        /* check if MKL_LONG is sufficient to hold dims */
        if (howmany != howmany_dims[0].n)
            goto broken;
        if (idistance != howmany_dims[0].is)
            goto broken;
        if (odistance != howmany_dims[0].os)
            goto broken;

        s = DftiSetValue(mkl_plan->desc, DFTI_NUMBER_OF_TRANSFORMS, howmany);
        if (BAD(s))
            goto broken;

        s = DftiSetValue(mkl_plan->desc, DFTI_INPUT_DISTANCE, idistance);
        if (BAD(s))
            goto broken;

        s = DftiSetValue(mkl_plan->desc, DFTI_OUTPUT_DISTANCE, odistance);
        if (BAD(s))
            goto broken;
    }

    if (fftw3_mkl.number_of_user_threads > 1)
    {
        s = DftiSetValue(mkl_plan->desc, DFTI_NUMBER_OF_USER_THREADS,
                         (MKL_LONG)fftw3_mkl.number_of_user_threads);
        if (BAD(s))
            goto broken;
    }

    {
        int nt = mkl_domain_get_max_threads(MKL_FFT);

        if (nt != fftw3_mkl.nthreads)
            mkl_domain_set_num_threads(fftw3_mkl.nthreads, MKL_FFT);
        s = DftiCommitDescriptor(mkl_plan->desc);
        if (nt != fftw3_mkl.nthreads)
            mkl_domain_set_num_threads(nt, MKL_FFT);
    }
    if (BAD(s))
        goto broken;

    return (fftw_plan)mkl_plan;

  broken:
    /* possibly report the reason before returning NULL */
    fftw3_mkl.delete_plan(mkl_plan);
    return NULL;
}

static void
execute_fi(fftw_mkl_plan p)
{
    DftiComputeForward(p->desc, p->io[0]);
}

static void
execute_fo(fftw_mkl_plan p)
{
    DftiComputeForward(p->desc, p->io[0], p->io[1]);
}
